/* demux -- simple demux tool for the Linux DVB S2 API
 *
 * UDL (updatelee@gmail.com)
 * Derived from work by:
 * 	Igor M. Liplianin (liplianin@me.by)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "demux.h"

#if DVB_API_VERSION < 5
#error tune-s2 requires Linux DVB driver API version 5.0 or newer!
#endif

#define BUFFY (188*20)
#define MAX_LENGTH (1024*1024*5) /* record 5MB */

char *usage =
	"\n"
	"   usage: demux apid vpid pcr pmt [options]\n"
	"          demux 1111 2222 333 444 [options]\n"
	"	-adapter N     : use given adapter (default 0)\n"
	"	-demux N       : use given demux (default 0)\n"
	"	-dvr N         : use given dvr (default 0)\n"
	"	-filename      : use filename insted of /tmp/test.ts\n"
	"	-help          : help\n";

int main(int argc, char *argv[])
{
	if (!argv[1] || strcmp(argv[1], "-h") == 0)
	{
		printf("%s", usage);
		return -1;
	}
	int apid	= strtoul(argv[1], NULL, 0);
	int vpid	= strtoul(argv[2], NULL, 0);
	int pcrpid	= strtoul(argv[3], NULL, 0);
	int pmtpid	= strtoul(argv[4], NULL, 0);

	char demux_devname[80];
	char dvr_devname[80];
	char *dvr_filename = "/tmp/test.ts";
	
	int adapter = 0, demux = 0, dvr = 0;

	int a;
	for( a = 5; a < argc; a++ )
	{
		if ( !strcmp(argv[a], "-adapter") )
			adapter = strtoul(argv[a+1], NULL, 0);
		if ( !strcmp(argv[a], "-demux") )
			demux = strtoul(argv[a+1], NULL, 0);
		if ( !strcmp(argv[a], "-dvr") )
			dvr = strtoul(argv[a+1], NULL, 0);
		if ( !strcmp(argv[a], "-filename") )
			dvr_filename = argv[a+1];
		if ( !strcmp(argv[a], "-help") )
		{
			printf("%s", usage);
			return -1;
		}
	}

	printf("apid: %d vpid: %d \n", apid, vpid);

	snprintf(demux_devname, 80, "/dev/dvb/adapter%d/demux%d", adapter, demux);
	printf("opening: %s\n", demux_devname);
	snprintf(dvr_devname, 80, "/dev/dvb/adapter%d/dvr%d", adapter, dvr);
	printf("opening: %s\n", dvr_devname);

	int i;
	int len;
	int written;
	uint8_t buf[BUFFY];
	uint64_t length;
	struct pollfd pfd[1];

	int dvr_fd;
	if ((dvr_fd = open(dvr_devname, O_RDONLY|O_NONBLOCK)) < 0)
	{
		printf("failed to open '%s': %d %m\n", dvr_devname, errno);
		return -1;
	}

	int demux1;
	if ((demux1 = open(demux_devname, O_RDWR|O_NONBLOCK)) < 0)
	{
		printf("failed to open '%s': %d %m\n", demux_devname, errno);
		return -1;
	}
	int demux2;
	if ((demux2 = open(demux_devname, O_RDWR|O_NONBLOCK)) < 0)
	{
		printf("failed to open '%s': %d %m\n", demux_devname, errno);
		return -1;
	}
	int demux3;
	if ((demux3 = open(demux_devname, O_RDWR|O_NONBLOCK)) < 0)
	{
		printf("failed to open '%s': %d %m\n", demux_devname, errno);
		return -1;
	}
	int demux4;
	if ((demux4 = open(demux_devname, O_RDWR|O_NONBLOCK)) < 0)
	{
		printf("failed to open '%s': %d %m\n", demux_devname, errno);
		return -1;
	}
	int demux5;
	if ((demux5 = open(demux_devname, O_RDWR|O_NONBLOCK)) < 0)
	{
		printf("failed to open '%s': %d %m\n", demux_devname, errno);
		return -1;
	}

	struct dmx_pes_filter_params pesFilterParams;

	pesFilterParams.pid = vpid;
	pesFilterParams.input = DMX_IN_FRONTEND; 
	pesFilterParams.output = DMX_OUT_TS_TAP; 
	pesFilterParams.pes_type = DMX_PES_VIDEO; 
	pesFilterParams.flags = DMX_IMMEDIATE_START;
	if (ioctl(demux1, DMX_SET_PES_FILTER, &pesFilterParams) == -1){
		perror("DEMUX1: DMX_SET_PES_FILTER");
		return -1; 
	}
	pesFilterParams.pid = apid; 
	pesFilterParams.input = DMX_IN_FRONTEND; 
	pesFilterParams.output = DMX_OUT_TS_TAP; 
	pesFilterParams.pes_type = DMX_PES_AUDIO; 
	pesFilterParams.flags = DMX_IMMEDIATE_START;
	if (ioctl(demux2, DMX_SET_PES_FILTER, &pesFilterParams) == -1){
		perror("DEMUX2: DMX_SET_PES_FILTER");
		return -1; 
	}
	pesFilterParams.pid = pcrpid; 
	pesFilterParams.input = DMX_IN_FRONTEND; 
	pesFilterParams.output = DMX_OUT_TS_TAP; 
	pesFilterParams.pes_type = DMX_PES_PCR; 
	pesFilterParams.flags = DMX_IMMEDIATE_START;
	if (ioctl(demux3, DMX_SET_PES_FILTER, &pesFilterParams) == -1){
		perror("DEMUX3: DMX_SET_PES_FILTER");
		return -1; 
	}
	pesFilterParams.pid = pmtpid; 
	pesFilterParams.input = DMX_IN_FRONTEND; 
	pesFilterParams.output = DMX_OUT_TS_TAP; 
	pesFilterParams.pes_type = DMX_PES_OTHER; 
	pesFilterParams.flags = DMX_IMMEDIATE_START;
	if (ioctl(demux4, DMX_SET_PES_FILTER, &pesFilterParams) == -1){
		perror("DEMUX3: DMX_SET_PES_FILTER");
		return -1; 
	}
	pesFilterParams.pid = 0; 
	pesFilterParams.input = DMX_IN_FRONTEND; 
	pesFilterParams.output = DMX_OUT_TS_TAP; 
	pesFilterParams.pes_type = DMX_PES_OTHER; 
	pesFilterParams.flags = DMX_IMMEDIATE_START;
	if (ioctl(demux5, DMX_SET_PES_FILTER, &pesFilterParams) == -1){
		perror("DEMUX3: DMX_SET_PES_FILTER");
		return -1; 
	}

	printf("Recording %2.0f MB of test file in TS format\n", MAX_LENGTH/(1024.0*1024.0));
	length = 0;

	/* open output file */
	int dvr_out;
	if ((dvr_out = open(dvr_filename, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)) < 0){
		printf("failed to open '%s': %d %m\n", dvr_filename, errno);
		return -1;
	}

	pfd[0].fd = dvr_fd;
	pfd[0].events = POLLIN;

    int c;
    set_conio_terminal_mode();
    do {
		if (poll(pfd,1,1))
		{
			if (pfd[0].revents & POLLIN)
			{
				len = read(dvr_fd, buf, BUFFY);
				if (len > 0)
				{
					written = 0;
					while (written < len) 
						written += write(dvr_out, buf, len);
					length += len;
					printf("written %2.0f MB\r", length/1024./1024.);
				} else {
					perror("recording");
					return -1;
				}
			}
		}
		if ( kbhit() )
	   		c = kbgetchar(); /* consume the character */
    } while( (char)c != 'q' );

	printf("Closing demux ... \n");
	close(dvr_out);
	close(dvr_fd);
	close(demux5);
	close(demux4);
	close(demux3);
	close(demux2);
	close(demux1);
	return 0;
}
